import { world, system } from "mojang-minecraft";
import { ActionFormData } from "mojang-minecraft-ui";
import { Ring } from "./lib/ring";
import { revengeRing } from "./lib/rings/revenge";
import { versatilityRing } from "./lib/rings/versatility";
import { vitalityRing } from "./lib/rings/vitality";

const rings = [ "divine:vitality_ring", "divine:versatility_ring", "divine:revenge_ring" ];

world.events.tick.subscribe((tickEvent) => { 
  if ((tickEvent.currentTick % 20) != 0) return;

  for (const player of world.getPlayers()) {
    const container = player.getComponent("inventory").container;
     
    for (const ringName of rings) {
      const ringItem = findItem(container, ringName);
      
      if (!ringItem.item) continue;
      const ring = new Ring(container, ringItem.item, ringItem.slot);
      const level = parseInt(ring.level);

      if (ringName == "divine:vitality_ring") {
        vitalityRing(player, level, ring.category);
      } else if (ringName == "divine:versatility_ring") {
        versatilityRing(player, level, ring.category);
      } else {
        revengeRing(player, level, ring.category);
      }
    }
  }
});

world.events.itemUse.subscribe((evt) => {
  const { source: player, item } = evt;

  if (player.id == "minecraft:player" && player.isSneaking) {
    const container = player.getComponent("inventory").container;

    if (!item || !rings.includes(item.id)) return;
    const ring = new Ring(container, item, player.selectedSlot);
    const buttons = [ "textures/ui/A_enabled.png", "textures/ui/B_enabled.png", "textures/ui/C_enabled.png" ];
    const levelXp = 10 + (parseInt(ring.level) * 5) + (parseInt(ring.level) == 3 ? 5 : 0);
    const upgradeForm = new ActionFormData()
      .title("Upgrade ring level")
      .body(`Xp level needed: ${hasXp(player, levelXp) ? "§a" : "§c"}${levelXp}§r`);

    // Uograde buttons dynamic textures
    for (const _ of buttons) {
      if (ring.category > "A") buttons[0] = "textures/ui/A_disabled";
      if (ring.category > "B" || ring.category < "B") buttons[1] = "textures/ui/B_disabled.png";
      if (parseInt(ring.level) == 1 && ring.category == "C" || ring.category < "C") buttons[2] = "textures/ui/C_disabled";
    }

    upgradeForm.button("Upgrade level A", buttons[0]);
    upgradeForm.button("Upgrade level B", buttons[1]);
    upgradeForm.button("Upgrade level C", buttons[2]);
    
    upgradeForm.show(player).then((response) => {
      if (response.canceled) return;
      else if (buttons[response.selection].includes("disabled")) {
        const category = [ "A", "B", "C" ][response.selection];

        if (category < ring.category) {
          sendTranslatedMessage(player, `divine.rings.upgrade.error.degrade`);
        } else {
          sendTranslatedMessage(player, `divine.rings.upgrade.error.notEnoughtPower`)
        }
      } else if (parseInt(ring.level) == 0 && ring.category == "C" && hasXp(player, 30)) {
        container.setItem(player.selectedSlot, ring.upgrade());
        removeXp(player, 30);
        sendTranslatedMessage(player, `divine.rings.upgrade.final`);
      } else if (parseInt(ring.level) > 0 && ring.category == "C") {
        sendTranslatedMessage(player, `divine.rings.upgrade.error.final`);
      } else if (ring.category < "C" && hasXp(player, levelXp)) {
        removeXp(player, levelXp);
        container.setItem(player.selectedSlot, ring.upgrade());
        sendTranslatedMessage(player, `divine.rings.upgrade`, [ ` ${parseInt(ring.level)}-${ring.category}` ]);
      } else {
        sendTranslatedMessage(player, `divine.rings.upgrade.error.noXp`);
      }
    });
  }
});

system.events.beforeWatchdogTerminate.subscribe((evt) => {
   evt.cancel = true;
});

function findItem(container, identifier) {
  function getItems() {
    const items = [];

    for (let i = 0; i < container.size; i++) {
      const item = container.getItem(i);

      if (!item) {
        items.push({ id: "minecraft:air", amount: 0, data: 0 });
        continue;
      }
      items.push(item);
    }
    return items;
  }
 
  return {
    item: getItems().find((item) => item.id === identifier),
    slot: getItems().findIndex((item) => item.id === identifier)
  };
}

function hasXp(player, xpLevel) {
  try {
    const playerXp = player.runCommand("xp 0 @s").level;

    return playerXp >= xpLevel;
  } catch {}
  return false;
}

function removeXp(player, xpLevel) {
  try {
    player.runCommand(`xp ${-xpLevel}L @s`);
  } catch {}
}

function sendTranslatedMessage(player, lang, extraMessages = []) {
  try {
    if (extraMessages.length > 0) {
      const raw = [ { translate: lang }, ...extraMessages.map((message) => { return { "text": message } }) ];

      return player.runCommand(`tellraw @s { "rawtext": ${JSON.stringify(raw)} }`);
    }
    player.runCommand(`tellraw @s { "rawtext": [{ "translate": "${lang}" }] }`);
  } catch {}
}
